
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <glib.h>
#include <string.h>


#include "error.h"
#include "pixbuf.h"


struct _uic_pixbuf {
	GdkPixbuf *pb;
};


uic_pixbuf
*uic_pixbuf_new(gint bits_per_sample, gint width, gint height)
{
	GdkPixbuf *pb;
	uic_pixbuf *uic_pb;

	pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, bits_per_sample, width, height);

	if (pb == NULL) {
		uic_errnum = UIC_PIXBUF_ERR_NOMEM;
		uic_set_error("not enough mem");
		return NULL;
	}
	
	gdk_pixbuf_fill(pb, 0xffffffff);

	uic_pb = g_malloc(sizeof(*uic_pb));

	uic_pb->pb = pb;

	return uic_pb;
}

uic_pixbuf *
uic_pixbuf_new_from_file(const gchar *path)
{
	uic_pixbuf *uic_pb;

	GdkPixbuf *pb;
	GError *err = NULL;
	
	pb = gdk_pixbuf_new_from_file(path, &err);
	
	if (pb == NULL) {
		uic_errnum = UIC_PIXBUF_ERR_CREATE;
		uic_set_error(err->message);
		g_clear_error(&err);
		return NULL;
	}

	uic_pb = g_malloc(sizeof(*uic_pb));

	uic_pb->pb = pb;

	return uic_pb;
}

uic_pixbuf 
*uic_pixbuf_new_from_buffer(const gchar *buf, gint bits_per_sample, gint width, gint height, gint rowstride)
{
	uic_pixbuf *uic_pb;
	GdkPixbuf *pb;

	pb = gdk_pixbuf_new_from_data((guchar*)buf, GDK_COLORSPACE_RGB, FALSE, bits_per_sample, width,
	    height, rowstride, NULL, NULL);


	uic_pb = g_malloc(sizeof(*uic_pb));

	uic_pb->pb = pb;

	return uic_pb;
}

gint
uic_pixbuf_get_width(const uic_pixbuf *src)
{
	return (gint)gdk_pixbuf_get_width(src->pb);
}

gint
uic_pixbuf_get_height(const uic_pixbuf *src)
{
	return (gint)gdk_pixbuf_get_height(src->pb);
}

gint 
uic_pixbuf_get_bits_per_sample(const uic_pixbuf *src)
{
	return (gint)gdk_pixbuf_get_bits_per_sample(src->pb);
}

gint
uic_pixbuf_get_rowstride(const uic_pixbuf *src)
{
	return (gint)gdk_pixbuf_get_rowstride(src->pb);
}


gint
uic_pixbuf_get_buffer_size(const uic_pixbuf *src)
{
	return uic_pixbuf_get_height(src) * uic_pixbuf_get_rowstride(src);
}

/*
 * Create duplicate of src and return pointer to it
 * If copy fails, returns NULL;
 */
uic_pixbuf *
uic_pixbuf_copy(const uic_pixbuf *src)
{
	GdkPixbuf *pb;
	uic_pixbuf *uic_pb;

	pb = gdk_pixbuf_copy(src->pb);
	
	if (pb == NULL) {
		uic_errnum = UIC_PIXBUF_ERR_NOMEM;
		uic_set_error("not enough mem");
		return NULL;
	}
	
	uic_pb = g_malloc(sizeof(*uic_pb));
	
	uic_pb->pb = pb;

	return uic_pb;
}



/*
 * Resize src and store resized uic_pixbuf into dst( WARNING: you must initialize dst by uic_pixbuf_new)
 */
void
uic_pixbuf_resize(uic_pixbuf *dst, const uic_pixbuf *src, gint dst_width, gint dst_height)
{
	gint src_width, src_height;

	src_width = uic_pixbuf_get_width(src);
	src_height = uic_pixbuf_get_height(src);

	gdk_pixbuf_scale(src->pb, dst->pb, 0, 0, dst_width, dst_height,
	    0, 0, (double) dst_width/src_width, (double) dst_height/src_height, GDK_INTERP_NEAREST);

}

/*
 * Cut area from src and store it to into dst( WARNING: you must initialize dst by uic_pixbuf_new)
 */
void
uic_pixbuf_cut_area(uic_pixbuf *dst, const uic_pixbuf *src, gint x, gint y, gint width, gint height)
{	
	gdk_pixbuf_scale(src->pb, dst->pb, x, y, width, height,
	    0, 0, 1, 1, GDK_INTERP_NEAREST);
}

gint
uic_pixbuf_save_to_file(const uic_pixbuf *pb, const gchar *path, const gchar *type)
{
	GError *err = NULL;

	if (gdk_pixbuf_save(pb->pb, path, type, &err, NULL) == FALSE) {
		uic_errnum = UIC_PIXBUF_ERR_SAVE;
		uic_set_error(err->message);
		g_clear_error(&err);
		return -1;
	}
	return 0;
}

gint
uic_pixbuf_save_to_buffer(const uic_pixbuf *pb, gchar *buf)
{
	guchar *pix;
	gint buf_sz;

	buf_sz = uic_pixbuf_get_buffer_size(pb);

	pix = gdk_pixbuf_get_pixels(pb->pb);

	g_memmove(buf, pix, buf_sz);

	return buf_sz;
}

void
uic_pixbuf_delete(uic_pixbuf *pb)
{
	g_object_unref(pb->pb);
	g_free(pb);
}

